/*
 * NAME:
 *	crl - check for revoked x509 certs
 *
 * DESCRIPTION:
 *	SSL Verify callbacks can call crl_check(ctx,xs), which will
 *	indicate which of the following cases is true.
 *
 *	1	we cannot find a CRL for issuer, return 0
 *	2	we have a CRL which revokes xs, return -2
 *	3	we have an expired CRL for issuer which does not revoke xs,
 *		return -1
 *	4	we have a valid CRL for issuer which does not revoke xs,
 *		return 1
 *
 * SEE ALSO:
 *	sslfd(3)
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@crufty.net>
 */
/*
 *      @(#)Copyright (c) 1996 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@crufty.net
 */
#ifndef lint
static char RCSid[] = "$Id: crl.c,v 1.13 2002/12/13 23:08:32 sjg Exp $";
#endif

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <stdio.h>
#if defined(HAVE_STDLIB_H)
#include <stdlib.h>
#endif
#include <sys/types.h>
#if defined(HAVE_SYS_SOCKET_H)
#include <sys/socket.h>
#endif
#include <netinet/in.h>
#if defined(HAVE_SYSLOG_H)
#include <syslog.h>
#endif
#include <string.h>
#if defined(HAVE_UNISTD_H)
#include <unistd.h>
#endif
#ifndef R_OK
#include <sys/file.h>
#endif

#include "openssl/buffer.h"
#include "openssl/x509.h"
#include "openssl/pem.h"
#include "openssl/ssl.h"

#ifdef libsslfd
# include "sslfd.h"
#else
# include "openssl/crypto.h"
# include "openssl/x509_vfy.h"
#endif

X509_CRL *
find_crl(ctx, xs)
	X509_STORE_CTX *ctx;
	X509 *xs;
{
	X509_OBJECT obj;
	X509_NAME *issuer;
	X509_CRL *crl = 0;
	u_long h;
	int i, k;
	char buf[256];
	
	if (!ctx)
		return 0;		/* should/can not happen */

	issuer = X509_get_issuer_name(xs);
	/*
	 * REVISIT: is there any point checking a CRL for a
	 * self-signed cert?  In any case we need to do this for 0.9
	 * so that we do get the CRL ok for the subject cert.
	 */
	if (X509_NAME_cmp(X509_get_subject_name(xs),issuer) == 0)
		return 0;
	
	memset((char *) &obj, 0, sizeof (obj));	/* initialize it! */
	if (X509_STORE_get_by_subject(ctx,X509_LU_CRL,issuer,&obj) > 0) 
		return (obj.data.crl);
	
	X509_OBJECT_free_contents(&obj);
	return 0;
}

int
asn1cmp(a, b)
	ASN1_INTEGER *a, *b;
{
	int ret;

	if ((ret = a->length - b->length) == 0) {
		if ((ret = memcmp(a->data, b->data, a->length)) == 0) {
			ret = a->type - b->type;
		}
	}
	return ret;
}
	
/*
 * The possible cases are:
 * 1	we cannot find a CRL for issuer, return 0
 * 2	we have a CRL which revokes xs, return -2
 * 3	we have an expired CRL for issuer which does not revoke xs,
 * 	return -1
 * 4	we have a valid CRL for issuer which does not revoke xs,
 * 	return 1
 */

int
check_crl(ctx, xs)
	X509_STORE_CTX *ctx;
	X509 *xs;
{
	X509_CRL *crl = find_crl(ctx, xs);
	X509_REVOKED *r;
	ASN1_UTCTIME *utc;
	int ret = 1;				/* OK */
	
	if (!crl)
		return 0;			/* no CRL */
	
	while (ret > 0 &&
	       (r = (X509_REVOKED *) sk_pop((STACK *)crl->crl->revoked)) != NULL) {
		if (asn1cmp(xs->cert_info->serialNumber, r->serialNumber) == 0)
			ret = -2;		/* revoked */
	}
	/*
	 * not revoked, need to check if crl has expired
	 */
	if (ret > 0 && crl->crl->nextUpdate != 0 && /* its optional */
	    X509_cmp_current_time(crl->crl->nextUpdate) < 0)
		ret = -1;		/* expired */

	return ret;
}
